==> morphdb_utils-0.2.49.tar.gz: morphdb_utils-0.2.49/core/morph/task/utils/connections/connector.py <==
from typing import Optional, Union, cast

import pandas as pd

from morph.task.utils.connections.bigquery.api import BigqueryApi
from morph.task.utils.connections.bigquery.usecase import BigqueryUsecase
from morph.task.utils.connections.database.mysql import MysqlConnector
from morph.task.utils.connections.database.postgres import PostgresqlConnector
from morph.task.utils.connections.database.redshift import RedshiftConnector
from morph.task.utils.connections.snowflake.api import SnowflakeApi
from morph.task.utils.connections.snowflake.usecase import SnowflakeUsecase
from morph.task.utils.profile import (
    BigqueryConnectionOAuth,
    BigqueryConnectionServiceAccount,
    BigqueryConnectionServiceAccountJson,
    MysqlConnection,
    PostgresqlConnection,
    ProfileYaml,
    RedshiftConnection,
    SnowflakeConnectionKeyPair,
    SnowflakeConnectionKeyPairFile,
    SnowflakeConnectionOAuth,
    SnowflakeConnectionUserPassword,
)


class Connector:
    """
    Connector class to execute SQL queries on different databases
    @param connection_slug: Connection slug
    @param connection: Connection object (required to set value on cloud)
    @param is_cloud: Boolean to check if the connection is cloud or not
    """

    def __init__(
        self,
        connection_slug: str,
        connection: Optional[
            Union[
                PostgresqlConnection,
                MysqlConnection,
                RedshiftConnection,
                SnowflakeConnectionUserPassword,
                SnowflakeConnectionOAuth,
                SnowflakeConnectionKeyPair,
                SnowflakeConnectionKeyPairFile,
                BigqueryConnectionOAuth,
                BigqueryConnectionServiceAccount,
                BigqueryConnectionServiceAccountJson,
            ]
        ] = None,
        is_cloud: Optional[bool] = False,
    ):
        if connection is None:
            connection = ProfileYaml.find_connection(connection_slug)
        if connection is None:
            raise FileNotFoundError("No connections found in profiles.yaml")
        self.connection = connection
        self.is_cloud = is_cloud

    def execute_sql(self, sql: str) -> pd.DataFrame:
        if isinstance(self.connection, PostgresqlConnection):
            pg_db = PostgresqlConnector(
                self.connection,
                use_ssh=self.connection.ssh_host is not None
                and self.connection.ssh_host != "",
            )
            result = pg_db.execute_sql(sql)
            return pd.DataFrame(result.fetchall(), columns=result.keys())
        elif isinstance(self.connection, MysqlConnection):
            mysql_db = MysqlConnector(
                self.connection,
                use_ssh=self.connection.ssh_host is not None
                and self.connection.ssh_host != "",
            )
            result = mysql_db.execute_sql(sql)
            return pd.DataFrame(result.fetchall(), columns=result.keys())
        elif isinstance(self.connection, RedshiftConnection):
            redshift_db = RedshiftConnector(
                self.connection,
                use_ssh=self.connection.ssh_host is not None
                and self.connection.ssh_host != "",
            )
            result = redshift_db.execute_sql(sql)
            return pd.DataFrame(result.fetchall(), columns=result.keys())
        elif isinstance(self.connection, SnowflakeConnectionUserPassword):
            result, columns = SnowflakeApi.execute_sql_with_basic(
                sql=sql,
                username=self.connection.user,
                password=self.connection.password,
                account=self.connection.account,
                database=self.connection.database,
                schema=self.connection.schema,
                warehouse=self.connection.warehouse,
            )
            return pd.DataFrame(result, columns=columns)
        elif isinstance(self.connection, SnowflakeConnectionOAuth):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                tokens = SnowflakeApi.refresh_access_token(
                    refresh_token=self.connection.refresh_token,
                    account=self.connection.account,
                    client_id=self.connection.client_id,
                    client_secret=self.connection.client_secret,
                    redirect_uri=self.connection.redirect_uri,
                    code_verifier=self.connection.code_verifier,
                )
                access_token = cast(str, tokens["access_token"])
            result = SnowflakeUsecase(
                account=self.connection.account,
                access_token=access_token,
                database=self.connection.database,
                schema=self.connection.schema,
                warehouse=self.connection.warehouse,
                role=self.connection.role,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, SnowflakeConnectionKeyPair):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                access_token = SnowflakeApi.get_key_pair_access_token(
                    account_identifier=self.connection.account,
                    username=self.connection.username,
                    private_key_raw=self.connection.key_pair,
                    passphrase=self.connection.passphrase,
                )
            result = SnowflakeUsecase(
                account=self.connection.account,
                access_token=access_token,
                database=self.connection.database,
                schema=self.connection.schema,
                warehouse=self.connection.warehouse,
                role=self.connection.role,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, SnowflakeConnectionKeyPairFile):
            access_token = SnowflakeApi.get_key_pair_access_token(
                account_identifier=self.connection.account,
                username=self.connection.username,
                private_key_raw=self.connection.key_pair_path,
                passphrase=self.connection.passphrase,
            )
            result = SnowflakeUsecase(
                account=self.connection.account,
                access_token=access_token,
                database=self.connection.database,
                schema=self.connection.schema,
                warehouse=self.connection.warehouse,
                role=self.connection.role,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, BigqueryConnectionOAuth):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                tokens = BigqueryApi.refresh_access_token(
                    client_id=self.connection.client_id,
                    client_secret=self.connection.client_secret,
                    refresh_token=self.connection.refresh_token,
                )
                access_token = cast(str, tokens["access_token"])
            result = BigqueryUsecase(
                project_id=self.connection.project,
                access_token=access_token,
                location=self.connection.location,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, BigqueryConnectionServiceAccount):
            access_token = BigqueryApi.get_access_token_from_service_account(
                service_account_info=self.connection.keyfile
            )
            result = BigqueryUsecase(
                project_id=self.connection.project,
                access_token=access_token,
                location=self.connection.location,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, BigqueryConnectionServiceAccountJson):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                access_token = BigqueryApi.get_access_token_from_service_account(
                    service_account_info=self.connection.keyfile_json.to_dict()
                )
            result = BigqueryUsecase(
                project_id=self.connection.project,
                access_token=access_token,
                location=self.connection.location,
            ).query(sql)
            return pd.DataFrame(result)
        raise NotImplementedError("connection type not implemented")

==> morphdb_utils-0.2.53.tar.gz: morphdb_utils-0.2.53/core/morph/task/utils/connections/connector.py <==
from typing import Optional, cast

import pandas as pd

from morph.task.utils.connections.bigquery.api import BigqueryApi
from morph.task.utils.connections.bigquery.usecase import BigqueryUsecase
from morph.task.utils.connections.database.mysql import MysqlConnector
from morph.task.utils.connections.database.postgres import PostgresqlConnector
from morph.task.utils.connections.database.redshift import RedshiftConnector
from morph.task.utils.connections.snowflake.api import SnowflakeApi
from morph.task.utils.connections.snowflake.usecase import SnowflakeUsecase
from morph.task.utils.profile import (
    BigqueryConnectionOAuth,
    BigqueryConnectionServiceAccount,
    BigqueryConnectionServiceAccountJson,
    Connection,
    MysqlConnection,
    PostgresqlConnection,
    ProfileYaml,
    RedshiftConnection,
    SnowflakeConnectionKeyPair,
    SnowflakeConnectionKeyPairFile,
    SnowflakeConnectionOAuth,
    SnowflakeConnectionUserPassword,
)


class Connector:
    """
    Connector class to execute SQL queries on different databases
    @param connection_slug: Connection slug
    @param connection: Connection object (required to set value on cloud)
    @param is_cloud: Boolean to check if the connection is cloud or not
    """

    def __init__(
        self,
        connection_slug: str,
        connection: Optional[Connection] = None,
        is_cloud: Optional[bool] = False,
    ):
        if connection is None:
            connection = ProfileYaml.find_connection(None, connection_slug)
        if connection is None:
            raise FileNotFoundError("No connections found in profiles.yaml")
        self.connection = connection
        self.is_cloud = is_cloud

    def execute_sql(self, sql: str) -> pd.DataFrame:
        if isinstance(self.connection, PostgresqlConnection):
            pg_db = PostgresqlConnector(
                self.connection,
                use_ssh=self.connection.ssh_host is not None
                and self.connection.ssh_host != "",
            )
            result = pg_db.execute_sql(sql)
            return pd.DataFrame(result.fetchall(), columns=result.keys())
        elif isinstance(self.connection, MysqlConnection):
            mysql_db = MysqlConnector(
                self.connection,
                use_ssh=self.connection.ssh_host is not None
                and self.connection.ssh_host != "",
            )
            result = mysql_db.execute_sql(sql)
            return pd.DataFrame(result.fetchall(), columns=result.keys())
        elif isinstance(self.connection, RedshiftConnection):
            redshift_db = RedshiftConnector(
                self.connection,
                use_ssh=self.connection.ssh_host is not None
                and self.connection.ssh_host != "",
            )
            result = redshift_db.execute_sql(sql)
            return pd.DataFrame(result.fetchall(), columns=result.keys())
        elif isinstance(self.connection, SnowflakeConnectionUserPassword):
            result, columns = SnowflakeApi.execute_sql_with_basic(
                sql=sql,
                username=self.connection.user,
                password=self.connection.password,
                account=self.connection.account,
                database=self.connection.database,
                schema=self.connection.schema_,
                warehouse=self.connection.warehouse,
            )
            return pd.DataFrame(result, columns=columns)
        elif isinstance(self.connection, SnowflakeConnectionOAuth):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                tokens = SnowflakeApi.refresh_access_token(
                    refresh_token=self.connection.refresh_token,
                    account=self.connection.account,
                    client_id=self.connection.client_id,
                    client_secret=self.connection.client_secret,
                    redirect_uri=self.connection.redirect_uri,
                    code_verifier=self.connection.code_verifier,
                )
                access_token = cast(str, tokens["access_token"])
            result = SnowflakeUsecase(
                account=self.connection.account,
                access_token=access_token,
                database=self.connection.database,
                schema=self.connection.schema_,
                warehouse=self.connection.warehouse,
                role=self.connection.role,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, SnowflakeConnectionKeyPair):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                access_token = SnowflakeApi.get_key_pair_access_token(
                    account_identifier=self.connection.account,
                    username=self.connection.username,
                    private_key_raw=self.connection.key_pair,
                    passphrase=self.connection.passphrase,
                )
            result = SnowflakeUsecase(
                account=self.connection.account,
                access_token=access_token,
                database=self.connection.database,
                schema=self.connection.schema_,
                warehouse=self.connection.warehouse,
                role=self.connection.role,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, SnowflakeConnectionKeyPairFile):
            access_token = SnowflakeApi.get_key_pair_access_token(
                account_identifier=self.connection.account,
                username=self.connection.username,
                private_key_raw=self.connection.key_pair_path,
                passphrase=self.connection.passphrase,
            )
            result = SnowflakeUsecase(
                account=self.connection.account,
                access_token=access_token,
                database=self.connection.database,
                schema=self.connection.schema_,
                warehouse=self.connection.warehouse,
                role=self.connection.role,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, BigqueryConnectionOAuth):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                tokens = BigqueryApi.refresh_access_token(
                    client_id=self.connection.client_id,
                    client_secret=self.connection.client_secret,
                    refresh_token=self.connection.refresh_token,
                )
                access_token = cast(str, tokens["access_token"])
            result = BigqueryUsecase(
                project_id=self.connection.project,
                access_token=access_token,
                location=self.connection.location,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, BigqueryConnectionServiceAccount):
            access_token = BigqueryApi.get_access_token_from_service_account(
                service_account_info=self.connection.keyfile
            )
            result = BigqueryUsecase(
                project_id=self.connection.project,
                access_token=access_token,
                location=self.connection.location,
            ).query(sql)
            return pd.DataFrame(result)
        elif isinstance(self.connection, BigqueryConnectionServiceAccountJson):
            if self.is_cloud:
                access_token = self.connection.access_token
                if access_token is None:
                    raise ValueError("Access token is required for cloud connection")
            else:
                access_token = BigqueryApi.get_access_token_from_service_account(
                    service_account_info=self.connection.keyfile_json.model_dump()
                )
            result = BigqueryUsecase(
                project_id=self.connection.project,
                access_token=access_token,
                location=self.connection.location,
            ).query(sql)
            return pd.DataFrame(result)
        raise NotImplementedError("connection type not implemented")
